/******** <lalVerbatim file="BlockConditionDataCV"> ********
Author: Tibbits, M. M.
$Id: BlockConditionData.c,v 1.1 2002/01/24 23:08:04 jolien Exp $
********* </lalVerbatim> ********/

#include "BlockConditionData.h"
#include "BlockApplySearch.h"
#include <lal/SeqFactories.h>

NRCSID( BLOCKCONDITIONDATAC, "$Id: BlockConditionData.c,v 1.1 2002/01/24 23:08:04 jolien Exp $");

void LALConditionData
(
	LALStatus	*status,
	LALSearchInput	*inout,
	void		*searchParams,
	LALMPIParams	*mpiParams
)
{
	/*  Variable Declarations  */
	BlockSearchParams	*myParams;

	/*  Initialize status to pass to other functions  */
	INITSTATUS( status, "ConditionData", BLOCKCONDITIONDATAC );
	ATTATCHSTATUSPTR (status);

	/*  Sanity Checks on Input  */
	ASSERT( inout, status, BLOCKCONDITIONDATAH_ENULL, BLOCKCONDITIONDATAH_MSGENULL);

	/*  Variable Initialization  */
	myParams = ((BlockSearchParams*)(searchParams));

	/*  Get rank  */
	MPI_Comm_rank(*(mpiParams->comm), &(myParams->rank));

	/*  Start Master/Slave version  */
	/*  Which node is this running on?  */
	if (myParams->searchMaster)
	{
		/* master */
		DistributeDataMaster(status->statusPtr, inout, searchParams, mpiParams );
	}
	else
	{
		/* slave */
		DistributeDataSlave(status->statusPtr, inout, searchParams, mpiParams );
	}

	DETATCHSTATUSPTR( status );
	RETURN( status );
}

void DistributeDataMaster
(
	LALStatus	*status,
	LALSearchInput	*inout,
	void		*searchParams,
	LALMPIParams	*mpiParams
)
{
	/*  Variable Declarations  */
	BlockSearchParams	*myParams;
	REAL8Vector		*tempVector;
	MPIMessage		mpiMsg;
	MPI_Comm		*comm;
	UINT4			currSlaves;
	UINT4			iterator;
	UINT4			segLength;
	UINT4			segLimit;
	INT4			whichMoment;

	/*  Initialize status to pass to other functions  */
	INITSTATUS( status, "DistributeDataMaster", BLOCKCONDITIONDATAC );
	ATTATCHSTATUSPTR (status);

	/*  Assert Correctness of input structures  */
	ASSERT(searchParams, status,BLOCKCONDITIONDATAH_ENULL, BLOCKCONDITIONDATAH_MSGENULL);

	ASSERT(inout, status,BLOCKCONDITIONDATAH_ENULL, BLOCKCONDITIONDATAH_MSGENULL);

	/*  Variable Initialization  */
	comm		= ((MPI_Comm*)(mpiParams->comm));
	myParams	= ((BlockSearchParams*)(searchParams));

	/* get the number of search slaves */
	MPI_Comm_size(*(mpiParams->comm), &(myParams->numSlaves));

	/* The number returned INCLUDES the search master, so subtract 1 */
	myParams->numSlaves -= 1;

	/* Create vector to store results */
	LALDCreateVector(status->statusPtr, &(myParams->res), myParams->numSlaves);

	whichMoment	= myParams->maxMoment;
	currSlaves	= myParams->numSlaves;
	segLength	= 0;
	iterator	= 0;
	segLimit	= 0;
	tempVector	= NULL;

	BlockParseData ( status->statusPtr, inout, searchParams );
	myParams->initialized = 1;

	while(currSlaves > 0 )
	{
		LALMPIRecvMsg( status->statusPtr, &mpiMsg, (*comm));
		CHECKSTATUSPTR( status );

		if ( mpiMsg.source == myParams->numSlaves)
		{
			segLength = myParams->segmentLength +  ( myParams->vec->length % myParams->numSlaves );
		}
		else
		{
			segLength = myParams->segmentLength;
		}

		LALMPISendUINT4( status->statusPtr, &(segLength), mpiMsg.source, (*comm));
		CHECKSTATUSPTR( status );

		LALDCreateVector(status->statusPtr, &tempVector, segLength );
		CHECKSTATUSPTR( status );

		segLimit = ((mpiMsg.source-1)*myParams->segmentLength);

		for( iterator = ( 0 + segLimit); iterator < ( tempVector->length + segLimit ); iterator++)
		{
			tempVector->data[iterator - segLimit] = myParams->vec->data[iterator];  /*  ((REAL8)(myParams->vec->data[currSlaves[0]*myParams->segmentLength+iterator])); */
		}

		LALMPISendREAL8Vector(status->statusPtr, tempVector, mpiMsg.source, (*comm));
		CHECKSTATUSPTR( status );

		LALMPISendINT4(status->statusPtr, &(whichMoment), mpiMsg.source, (*comm));
		CHECKSTATUSPTR( status );

		/*  Clean up temporary Mem each time used  */
		LALDDestroyVector(status->statusPtr, &tempVector);
		CHECKSTATUSPTR( status );
		tempVector	= NULL;
		currSlaves--;
	}

	DETATCHSTATUSPTR( status );
	RETURN( status );

}

void DistributeDataSlave
(
	LALStatus	*status,
	LALSearchInput	*inout,
	void		*searchParams,
	LALMPIParams	*mpiParams
)
{
	/*  Variable Declarations  */
	BlockSearchParams	*myParams;
	REAL8Vector		*local;
	MPIMessage		sendMsg;
	MPI_Comm		*comm;
	BOOLEAN			noData;

	/*  Initialize status to pass to other functions  */
	INITSTATUS( status, "DistributeDataSlave", BLOCKCONDITIONDATAC );
	ATTATCHSTATUSPTR (status);

	/*  Variable Initialization  */
	myParams	= (searchParams);
	comm		= ((MPI_Comm*)(mpiParams->comm));
	noData		= 1;
	local		= NULL;
	sendMsg.source	= myParams->rank;
	sendMsg.msg	= PassData;
	sendMsg.send	= 1;

	/* Create length 1 array to transmit/receive the result */
	LALDCreateVector(status->statusPtr, &(myParams->res), 1);
	myParams->initialized = 0;

	/*  Send Message to say I am ready  */
	LALMPISendMsg( status->statusPtr, &sendMsg, 0, (*comm));
	CHECKSTATUSPTR( status );	

	/*  How much Data?  */
	LALMPIRecvUINT4( status->statusPtr, &(myParams->segmentLength), 0, (*comm));
	CHECKSTATUSPTR (status);

	/*  Create new structure to hold local data  */
	LALDCreateVector(status->statusPtr, &local, myParams->segmentLength );
	CHECKSTATUSPTR (status);

	/*  Receive input data  */
	LALMPIRecvREAL8Vector(status->statusPtr, local, 0, (*comm));
	CHECKSTATUSPTR (status);

	LALMPIRecvINT4( status->statusPtr, &(myParams->maxMoment), 0, (*comm));
	CHECKSTATUSPTR( status );

	myParams->vec = local;

	DETATCHSTATUSPTR( status );
	RETURN( status );
}

void BlockParseData
(
	LALStatus	*status,
	LALSearchInput	*inout,
	void		*searchParams
)
{

	/*  Variable Declarations  */
	BlockSearchParams	*myParams;
	REAL8Vector		*tempVector;
	BOOLEAN			dataFound;
	UINT4			iterator;
	UINT4			segmentCount;
	UINT4			offset;
	INT4			count;

	/*  Initialize status to pass to other functions  */
	INITSTATUS( status, "BlockParseData", BLOCKCONDITIONDATAC );
	ATTATCHSTATUSPTR (status);

	/*  Variable Initialization  */
	myParams	= ((BlockSearchParams*)(searchParams));
	tempVector	= myParams->vec = NULL;

	offset			= 0;
	dataFound		= 0;
	segmentCount		= 0;

	/*  Check that input exists  */
	ASSERT(inout, status, BLOCKCONDITIONDATAH_ENULL, BLOCKCONDITIONDATAH_MSGENULL);

	/*  Check that input exists  */
	ASSERT(inout->sequences, status, BLOCKCONDITIONDATAH_ENULL, BLOCKCONDITIONDATAH_MSGENULL);

	/*  Check that we are only given as much data as we want  */
	ASSERT(inout->numberSequences == 1, status, BLOCKCONDITIONDATAH_ENMSQ, BLOCKCONDITIONDATAH_MSGENMSQ);

	/*  Must have count >= 2*np  */
	count =  inout->sequences[0].dimensions[0];

	/*  Create new structure to hold local data  */
	LALDCreateVector(status->statusPtr, &tempVector, count);
	CHECKSTATUSPTR (status);

	for ( iterator = 0; iterator < count; iterator++)
	{
		tempVector->data[iterator] = ((REAL8)(inout->sequences[0].data.int2s[iterator]));
	}

	myParams->times.startSec = (INT8) inout->sequences[0].range.dTime.startSec;
	myParams->times.startNan = (INT8) inout->sequences[0].range.dTime.startNan;
	myParams->times.stopSec = (INT8) inout->sequences[0].range.dTime.stopSec;
	myParams->times.stopNan = (INT8) inout->sequences[0].range.dTime.stopNan;
	myParams->times.timeStepSize = (INT8) inout->sequences[0].range.dTime.timeStepSize;
	myParams->times.numberSamples = (INT8) inout->sequences[0].range.dTime.numberSamples;

	strcpy(myParams->channel, "H2");

	count /= (myParams->numSlaves);

	myParams->segmentLength = count;

	ASSERT( count >= 2, status, BLOCKCONDITIONDATAH_EFEW, BLOCKCONDITIONDATAH_MSGEFEW);

	myParams->vec = tempVector;

	DETATCHSTATUSPTR (status);
	RETURN(status);
}
