/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigFinalizeSearchCV"> ********
Author: Rahkola, R
$Id: ExtTrigFinalizeSearch.c,v 1.5 2002/04/18 19:39:20 rrahkola Exp $
********* </lalVerbatim> ********/

/* Include input/output functions if we're in the debug stage */
#ifdef DEBUG_EXTTRIG
#include <stdio.h>
#endif

/* standard C functions */
#include <math.h>

/* standard LAL functions*/
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>

/* LAL MPI interface */
#include <lal/Comm.h>

/* Associated headers */
#include <ExtTrig.h>
#include <ExtTrigFinalizeSearch.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGFINALIZESEARCHC, "$Id: ExtTrigFinalizeSearch.c,v 1.5 2002/04/18 19:39:20 rrahkola Exp $" );

/************************* wrapperAPI function definitions  ******************/

/*
 * Function LALFinalizeSearch
 *
 * Finalize routine does nothing but check arguments and free memory
 */
void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams)
{
  ETSearchParams* const params = (ETSearchParams *) *searchParams;

  INITSTATUS( status, "LALFinalizeSearch", EXTTRIGFINALIZESEARCHC );
  ATTATCHSTATUSPTR (status);

#ifdef DEBUG_EXTTRIG
  fprintf(stdout, "exttrig:LALFinalizeSearch - %s\n", params->id );
  fflush(stdout);
#endif

  /*
   *
   * clean up
   *
   */
  /* MASTER ONLY: free memory used by ETResult */
  if ( params->searchMaster ) {
    LALDestroyVector( status->statusPtr, &(params->result->xCorrLag) );
    CHECKSTATUSPTR( status );

    LALDestroyVector( status->statusPtr, &(params->result->xCorrDelay) );
    CHECKSTATUSPTR( status );

    LALDestroyVector( status->statusPtr, &(params->result->xCorrValue) );
    CHECKSTATUSPTR( status );
  }

  /* free memory used by RealFFTPlan */
  TRY( LALDestroyRealFFTPlan( status->statusPtr, &(params->fftPlan) ), 
       status );

  /* free memory used by ETDataSegmentVector */
  LALDestroyETDataSegmentVector( status->statusPtr, &(params->data) );
  CHECKSTATUSPTR( status );

  /* free memory used by ETSegList */
  LALDestroyETSegList( status->statusPtr, &(params->segments) );
  CHECKSTATUSPTR( status );

  /* free substructures of ETSearchParams */
  LALFree( params->result );
  params->result = NULL;

  LALFree( params->initParams );
  params->initParams = NULL;

  /* free ETSearchParams structure */
  LALFree( *searchParams );
  *searchParams = NULL;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

#ifdef __cplusplus
}
#endif
