/**** <lalVerbatim file="TFCInitSearchHV"> ***********
Author: Julien Sylvestre
$Id: InitSearch.h,v 1.9 2002/04/16 19:18:00 julien Exp $ 
***** </lalVerbatim> ***********************************/

#ifndef _INITSEARCH_H
#define _INITSEARCH_H

#include <LALWrapperInterface.h>
#include <lal/LALDatatypes.h>
#include <lal/LALRCSID.h>

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (INITSEARCHH, "$Id: InitSearch.h,v 1.9 2002/04/16 19:18:00 julien Exp $");

typedef enum {SNGL_BURST, GDS_TRIGGER} DbTable;

typedef struct tagTFCSearchParams {

  BOOLEAN searchMaster;
 
  UINT4 NData; /* total number of data to be searched */

  UINT4 Ttotal; /* total number of data in one segment */

  REAL8 T; /* time resolution */

  UINT4 fs; /* sampling frequency of channel */

  UINT4 olap; /* overlap (# of samples) between segments */

  /* data containers */
  REAL4 bpp; /* black pixel probability */

  REAL4 eGoal; /* absolute error goal on power threshld */

  REAL4* dmro;              /* container for the raw dataset */

  CListDir* dir;            /* container for the instructions */

  REAL4TimeSeries* tseries; /* container for the data to be analyzed by each node */

  TFPlaneParams* tspec;     /* container for the time-frequency parameters */

  DbTable table; /* database table name */

  char name[256]; /* channel name */

  REAL4* resp;   /* response function */

  REAL4* spec;   /* power spectrum */

  REAL4 Nsigma; /* number of std above mean to reject in second pass at estimation of mean and variance (Q and P0) */

  BOOLEAN WriteBlobs; /* set to 1 if pixel coordinates are to saved in sngl_transdata */

} TFCSearchParams;


/******** <lalErrTable file="TFCInitSearchHErrTab"> ********/
#define INITSEARCHH_ENULL     1
#define INITSEARCHH_ENNUL     2
#define INITSEARCHH_EALOC     3
#define INITSEARCHH_EARGS     4
#define INITSEARCHH_ESPOS      5
#define INITSEARCHH_ENAN      6
#define INITSEARCHH_EAOR      7
#define INITSEARCHH_EPOS      8
#define INITSEARCHH_EMMF      9
#define INITSEARCHH_ENAM      10
#define INITSEARCHH_ETAB      11

#define INITSEARCHH_MSGENULL     "Null pointer"
#define INITSEARCHH_MSGENNUL     "Non-null pointer"
#define INITSEARCHH_MSGEALOC     "Memory allocation error"
#define INITSEARCHH_MSGEARGS     "Wrong number of arguments"
#define INITSEARCHH_MSGESPOS      "Parameter <= 0 expected to be strictly positive"
#define INITSEARCHH_MSGENAN       "Numerical parameter is NaN"
#define INITSEARCHH_MSGEAOR      "Parameter expected to be in [0,1] is out of range"
#define INITSEARCHH_MSGEPOS      "Parameter < 0 when expected to be positive or zero"
#define INITSEARCHH_MSGEMMF      "Parameter minf > maxf"
#define INITSEARCHH_MSGENAM       "Channel/IFO name is zero length"
#define INITSEARCHH_MSGETAB       "Unknown table name"
/******** </lalErrTable> ********/


#ifdef  __cplusplus
}
#endif

#endif /* _INITSEARCH_H  */


